@@ -54,7 +54,7 @@ globalOptionsLong = ['no-prompts', 'generator=', 'verbose', 'make-gui']
 cmd_opt_dict = {
 	'about'     : ['', []],
 	'setup'     : ['g:', []],
-	'configure' : ['g:dr', ['debug', 'release']],
+	'configure' : ['g:dr', ['debug', 'release', 'game-device', 'vnc', 'mac-sdk=']],
 	'build'     : ['dr', ['debug', 'release']],
 	'clean'     : ['dr', ['debug', 'release']],
 	'update'    : ['', []],
@@ -69,6 +69,8 @@ cmd_opt_dict = {
 	'open'      : ['', []],
 	'genlist'   : ['', []],
 	'reset'	    : ['', []],
+	'signwin'	: ['', ['pfx=', 'pwd=', 'dist']],
+	'signmac'	: ['', ['identity=']]
 }
 
 # aliases to valid commands
