@@ -57,3 +57,10 @@ COSXClientTaskBarReceiver::getIcon() const
 {
 	return NULL;
 }
+
+IArchTaskBarReceiver*
+createTaskBarReceiver(const CBufferedLogOutputter* logBuffer)
+{
+	return new COSXClientTaskBarReceiver(logBuffer);
+}
+
