@@ -57,3 +57,9 @@ CXWindowsServerTaskBarReceiver::getIcon() const
 {
 	return NULL;
 }
+
+IArchTaskBarReceiver*
+createTaskBarReceiver(const CBufferedLogOutputter* logBuffer)
+{
+	return new CXWindowsServerTaskBarReceiver(logBuffer);
+}
