@@ -20,6 +20,7 @@
 #define ABOUTDIALOG__H
 
 #include <QDialog>
+#include "VersionChecker.h"
 
 #include "ui_AboutDialogBase.h"
 
@@ -32,6 +33,9 @@ class AboutDialog : public QDialog, public Ui::AboutDialogBase
 
 	public:
 		AboutDialog(QWidget* parent, const QString& synergyApp = QString());
+
+	private:
+		VersionChecker m_versionChecker;
 };
 
 #endif
