@@ -50,6 +50,12 @@ HICON						CArchMiscWindows::s_largeIcon = NULL;
 HICON						CArchMiscWindows::s_smallIcon = NULL;
 HINSTANCE					CArchMiscWindows::s_instanceWin32 = NULL;
 
+void
+CArchMiscWindows::cleanup()
+{
+	delete s_dialogs;
+}
+
 void
 CArchMiscWindows::init()
 {
