@@ -42,6 +42,20 @@ public:
 	*/
 	virtual std::string getPlatformName() const = 0;
 	//@}
+
+	//! Get a Synergy setting
+	/*!
+	Reads a Synergy setting from the system.
+	*/
+	virtual std::string setting(const std::string& valueName) const = 0;
+	//@}
+
+	//! Set a Synergy setting
+	/*!
+	Writes a Synergy setting from the system.
+	*/
+	virtual void setting(const std::string& valueName, const std::string& valueString) const = 0;
+	//@}
 };
 
 #endif
