@@ -30,8 +30,11 @@ enum ELevel {
 	kNOTE,			//!< For messages about notable events
 	kINFO,			//!< For informational messages
 	kDEBUG,			//!< For important debugging messages
-	kDEBUG1,		//!< For more detailed debugging messages
-	kDEBUG2			//!< For even more detailed debugging messages
+	kDEBUG1,		//!< For verbosity +1 debugging messages
+	kDEBUG2,		//!< For verbosity +2 debugging messages
+	kDEBUG3,		//!< For verbosity +3 debugging messages
+	kDEBUG4,		//!< For verbosity +4 debugging messages
+	kDEBUG5			//!< For verbosity +5 debugging messages
 };
 
 #endif
