@@ -33,7 +33,7 @@
 // CServerProxy
 //
 
-CServerProxy::CServerProxy(CClient* client, IStream* stream) :
+CServerProxy::CServerProxy(CClient* client, synergy::IStream* stream, IEventQueue& eventQueue) :
 	m_client(client),
 	m_stream(stream),
 	m_seqNum(0),
@@ -46,7 +46,8 @@ CServerProxy::CServerProxy(CClient* client, IStream* stream) :
 	m_ignoreMouse(false),
 	m_keepAliveAlarm(0.0),
 	m_keepAliveAlarmTimer(NULL),
-	m_parser(&CServerProxy::parseHandshakeMessage)
+	m_parser(&CServerProxy::parseHandshakeMessage),
+	m_eventQueue(eventQueue)
 {
 	assert(m_client != NULL);
 	assert(m_stream != NULL);
@@ -56,7 +57,7 @@ CServerProxy::CServerProxy(CClient* client, IStream* stream) :
 		m_modifierTranslationTable[id] = id;
 
 	// handle data on stream
-	EVENTQUEUE->adoptHandler(IStream::getInputReadyEvent(),
+	m_eventQueue.adoptHandler(m_stream->getInputReadyEvent(),
 							m_stream->getEventTarget(),
 							new TMethodEventJob<CServerProxy>(this,
 								&CServerProxy::handleData));
@@ -68,7 +69,7 @@ CServerProxy::CServerProxy(CClient* client, IStream* stream) :
 CServerProxy::~CServerProxy()
 {
 	setKeepAliveRate(-1.0);
-	EVENTQUEUE->removeHandler(IStream::getInputReadyEvent(),
+	m_eventQueue.removeHandler(m_stream->getInputReadyEvent(),
 							m_stream->getEventTarget());
 }
 
@@ -76,14 +77,14 @@ void
 CServerProxy::resetKeepAliveAlarm()
 {
 	if (m_keepAliveAlarmTimer != NULL) {
-		EVENTQUEUE->removeHandler(CEvent::kTimer, m_keepAliveAlarmTimer);
-		EVENTQUEUE->deleteTimer(m_keepAliveAlarmTimer);
+		m_eventQueue.removeHandler(CEvent::kTimer, m_keepAliveAlarmTimer);
+		m_eventQueue.deleteTimer(m_keepAliveAlarmTimer);
 		m_keepAliveAlarmTimer = NULL;
 	}
 	if (m_keepAliveAlarm > 0.0) {
 		m_keepAliveAlarmTimer =
-			EVENTQUEUE->newOneShotTimer(m_keepAliveAlarm, NULL);
-		EVENTQUEUE->adoptHandler(CEvent::kTimer, m_keepAliveAlarmTimer,
+			m_eventQueue.newOneShotTimer(m_keepAliveAlarm, NULL);
+		m_eventQueue.adoptHandler(CEvent::kTimer, m_keepAliveAlarmTimer,
 							new TMethodEventJob<CServerProxy>(this,
 								&CServerProxy::handleKeepAliveAlarm));
 	}
@@ -411,7 +412,8 @@ CServerProxy::translateKey(KeyID id) const
 		{ kKeyControl_L, kKeyControl_R },
 		{ kKeyAlt_L,     kKeyAlt_R },
 		{ kKeyMeta_L,    kKeyMeta_R },
-		{ kKeySuper_L,   kKeySuper_R }
+		{ kKeySuper_L,   kKeySuper_R },
+		{ kKeyAltGr,     kKeyAltGr}
 	};
 
 	KeyModifierID id2 = kKeyModifierIDNull;
@@ -447,6 +449,11 @@ CServerProxy::translateKey(KeyID id) const
 		side = 1;
 		break;
 
+	case kKeyAltGr:
+		id2 = kKeyModifierIDAltGr;
+		side = 1; // there is only one alt gr key on the right side
+		break;
+
 	case kKeyMeta_L:
 		id2  = kKeyModifierIDMeta;
 		side = 0;
@@ -485,14 +492,16 @@ CServerProxy::translateModifierMask(KeyModifierMask mask) const
 		KeyModifierControl,
 		KeyModifierAlt,
 		KeyModifierMeta,
-		KeyModifierSuper
+		KeyModifierSuper,
+		KeyModifierAltGr
 	};
 
 	KeyModifierMask newMask = mask & ~(KeyModifierShift |
 										KeyModifierControl |
 										KeyModifierAlt |
 										KeyModifierMeta |
-										KeyModifierSuper);
+										KeyModifierSuper |
+										KeyModifierAltGr );
 	if ((mask & KeyModifierShift) != 0) {
 		newMask |= s_masks[m_modifierTranslationTable[kKeyModifierIDShift]];
 	}
@@ -502,6 +511,9 @@ CServerProxy::translateModifierMask(KeyModifierMask mask) const
 	if ((mask & KeyModifierAlt) != 0) {
 		newMask |= s_masks[m_modifierTranslationTable[kKeyModifierIDAlt]];
 	}
+	if ((mask & KeyModifierAltGr) != 0) {
+		newMask |= s_masks[m_modifierTranslationTable[kKeyModifierIDAltGr]];
+	}
 	if ((mask & KeyModifierMeta) != 0) {
 		newMask |= s_masks[m_modifierTranslationTable[kKeyModifierIDMeta]];
 	}
@@ -864,6 +876,9 @@ CServerProxy::setOptions()
 		else if (options[i] == kOptionModifierMapForAlt) {
 			id = kKeyModifierIDAlt;
 		}
+		else if (options[i] == kOptionModifierMapForAltGr) {
+			id = kKeyModifierIDAltGr;
+		}
 		else if (options[i] == kOptionModifierMapForMeta) {
 			id = kKeyModifierIDMeta;
 		}
