@@ -38,11 +38,4 @@ extern const char* kVersion;
 // application version
 extern const char* kAppVersion;
 
-// exit codes
-static const int kExitSuccess   = 0;  // successful completion
-static const int kExitFailed    = 1;  // general failure
-static const int kExitTerminated  = 2;  // killed by signal
-static const int kExitArgs      = 3;  // bad arguments
-static const int kExitConfig    = 4;  // cannot read configuration
-
 #endif
