@@ -32,8 +32,11 @@
 #include "TMethodJob.h"
 #include "XArch.h"
 
+#include <math.h>
+
 #include <mach-o/dyld.h>
 #include <AvailabilityMacros.h>
+#include <IOKit/hidsystem/event_status_driver.h>
 
 // Set some enums for fast user switching if we're building with an SDK
 // from before such support was added.
@@ -84,7 +87,11 @@ COSXScreen::COSXScreen(bool isPrimary) :
 	m_pmWatchThread(NULL),
 	m_pmThreadReady(new CCondVar<bool>(m_pmMutex, false)),
 	m_activeModifierHotKey(0),
-	m_activeModifierHotKeyMask(0)
+	m_activeModifierHotKeyMask(0),
+	m_lastSingleClick(0),
+	m_lastDoubleClick(0),
+	m_lastSingleClickXCursor(0),
+	m_lastSingleClickYCursor(0)
 {
 	try {
 		m_displayID   = CGMainDisplayID();
@@ -92,11 +99,9 @@ COSXScreen::COSXScreen(bool isPrimary) :
 		m_screensaver = new COSXScreenSaver(getEventTarget());
 		m_keyState	  = new COSXKeyState();
 		
-		if (m_isPrimary) {
-			if (!AXAPIEnabled()) {
-				LOG((CLOG_ERR "Synergy server requires accessibility API enabled. Please check the option for \"Enable access for assistive devices\" in the Universal Access System Preferences panel. Unintentional key-replication will occur until this is fixed."));
-			}
-		}
+    // TODO: http://stackoverflow.com/questions/2950124/enable-access-for-assistive-device-programmatically
+		if (m_isPrimary && !AXAPIEnabled())
+			throw XArch("system setting not enabled: \"Enable access for assistive devices\"");
 		
 		// install display manager notification handler
 #if defined(MAC_OS_X_VERSION_10_5)
@@ -316,7 +321,7 @@ COSXScreen::registerHotKey(KeyID key, KeyModifierMask mask)
 	// get mac virtual key and modifier mask matching synergy key and mask
 	UInt32 macKey, macMask;
 	if (!m_keyState->mapSynergyHotKeyToMac(key, mask, macKey, macMask)) {
-		LOG((CLOG_WARN "could not map hotkey id=%04x mask=%04x", key, mask));
+		LOG((CLOG_DEBUG "could not map hotkey id=%04x mask=%04x", key, mask));
 		return 0;
 	}
 	
@@ -476,8 +481,22 @@ COSXScreen::postMouseEvent(CGPoint& pos) const
 }
 
 void
-COSXScreen::fakeMouseButton(ButtonID id, bool press) const
+COSXScreen::fakeMouseButton(ButtonID id, bool press)
 {
+	NXEventHandle handle = NXOpenEventStatus();
+	double clickTime = NXClickTime(handle);
+	
+	if ((ARCH->time() - m_lastDoubleClick) <= clickTime) {
+		// drop all down and up fakes immedately after a double click.
+		// TODO: perhaps there is a better way to do this, usually in
+		// finder, if you tripple click a folder, it will open it and
+		// then select a folder under the cursor -- and perhaps other
+		// strange behaviour might happen?
+		LOG((CLOG_DEBUG1 "dropping mouse button %s",
+			press ? "press" : "release"));
+		return;
+	}
+	
 	// Buttons are indexed from one, but the button down array is indexed from zero
 	UInt32 index = id - kButtonLeft;
 	if (index >= NumButtonIDs) {
@@ -491,36 +510,71 @@ COSXScreen::fakeMouseButton(ButtonID id, bool press) const
 	}
 	pos.x = m_xCursor;
 	pos.y = m_yCursor;
+
+	// variable used to detect mouse coordinate differences between
+	// old & new mouse clicks. Used in double click detection.
+	SInt32 xDiff = m_xCursor - m_lastSingleClickXCursor;
+	SInt32 yDiff = m_yCursor - m_lastSingleClickYCursor;
+	double diff = sqrt(xDiff * xDiff + yDiff * yDiff);
+	// max sqrt(x^2 + y^2) difference allowed to double click
+	// since we don't have double click distance in NX APIs
+	// we define our own defaults.
+	const double maxDiff = sqrt(2) + 0.0001;
+
+	if (press && (id == kButtonLeft) &&
+		((ARCH->time() - m_lastSingleClick) <= clickTime) &&
+		diff <= maxDiff) {
+
+		LOG((CLOG_DEBUG1 "faking mouse left double click"));
+		
+		// finder does not seem to detect double clicks from two separate
+		// CGEventCreateMouseEvent calls. so, if we detect a double click we
+		// use CGEventSetIntegerValueField to tell the OS.
+		// 
+		// the caveat here is that findor will see this as a single click 
+		// followed by a double click (even though there should be only a
+		// double click). this may cause weird behaviour in other apps.
+		//
+		// for some reason using the old CGPostMouseEvent function, doesn't
+		// cause double clicks (though i'm sure it did work at some point).
+		
+		CGEventRef event = CGEventCreateMouseEvent(
+			NULL, kCGEventLeftMouseDown, pos, kCGMouseButtonLeft);
+		
+		CGEventSetIntegerValueField(event, kCGMouseEventClickState, 2);
+		m_buttonState.set(index, kMouseButtonDown);
+		CGEventPost(kCGHIDEventTap, event);
+		
+		CGEventSetType(event, kCGEventLeftMouseUp);
+		m_buttonState.set(index, kMouseButtonUp);
+		CGEventPost(kCGHIDEventTap, event);
+		
+		CFRelease(event);
 	
-	// synthesize event.  CGEventCreateMouseEvent creates a retained mouse
-	// event, which must also be posted and released. Note this is
-	// similar to the use of CGEventRef in postMouseEvent above.
-	// One of the arguments changes based on whether a button is being
-	// pressed or released, pressed corresponding to when "press" is true.
-	CGEventRef event;
-	
-	// the switch statement handles which button was pressed.  the left
-	// and right mouse buttons must be handled separately from any
-	// other buttons
-	
-	CGEventType type;
-	MouseButtonState state;
-	if (press) { 
-		state = kMouseButtonDown;
-	} else {
-		state = kMouseButtonUp;
+		m_lastDoubleClick = ARCH->time();
 	}
-
-	MouseButtonEventMapType thisButtonMap = MouseButtonEventMap[index];
-	type = thisButtonMap[state];
-
-	event = CGEventCreateMouseEvent(NULL, type, pos, index);
-
-	m_buttonState.set(index, state);
-
-	CGEventPost(kCGHIDEventTap, event);
+	else {
+		
+		// ... otherwise, perform a single press or release as normal.
+		
+		MouseButtonState state = press ? kMouseButtonDown : kMouseButtonUp;
+		
+		LOG((CLOG_DEBUG1 "faking mouse button %s", press ? "press" : "release"));
+		
+		MouseButtonEventMapType thisButtonMap = MouseButtonEventMap[index];
+		CGEventType type = thisButtonMap[state];
+		
+		CGEventRef event = CGEventCreateMouseEvent(NULL, type, pos, index);
 	
-	CFRelease(event);
+		m_buttonState.set(index, state);
+		CGEventPost(kCGHIDEventTap, event);
+		
+		CFRelease(event);
+	
+		m_lastSingleClick = ARCH->time();
+		m_lastSingleClickXCursor = m_xCursor;
+		m_lastSingleClickYCursor = m_yCursor;
+	}
 }
 
 void
@@ -588,12 +642,11 @@ COSXScreen::fakeMouseWheel(SInt32 xDelta, SInt32 yDelta) const
 void
 COSXScreen::showCursor()
 {
+	LOG((CLOG_DEBUG "showing cursor"));
     CGDisplayShowCursor(m_displayID);
     CFStringRef propertyString = CFStringCreateWithCString(NULL, "SetsCursorInBackground", kCFStringEncodingMacRoman);
     CGSSetConnectionProperty(_CGSDefaultConnection(), _CGSDefaultConnection(), propertyString, kCFBooleanFalse);
     CFRelease(propertyString);
-	
-    LOG((CLOG_DEBUG "Trying to show cursor."));
 }
 
 void
@@ -603,8 +656,8 @@ COSXScreen::hideCursor()
     CGSSetConnectionProperty(_CGSDefaultConnection(), _CGSDefaultConnection(), propertyString, kCFBooleanTrue);
     CFRelease(propertyString);
 
+	LOG((CLOG_DEBUG "hiding cursor"));
     CGDisplayHideCursor(m_displayID);
-    LOG((CLOG_DEBUG "Trying to hide cursor."));
 }
 
 void
@@ -646,11 +699,11 @@ COSXScreen::enable()
 										this);
 	}
 	if(!m_eventTapPort) {
-		LOG((CLOG_ERR "Failed to create quartz event tap."));
+		LOG((CLOG_ERR "failed to create quartz event tap"));
 	}
 	m_eventTapRLSR=CFMachPortCreateRunLoopSource(kCFAllocatorDefault, m_eventTapPort, 0);
 	if(!m_eventTapRLSR) {
-		LOG((CLOG_ERR "Failed to create a CFRunLoopSourceRef for the quartz event tap."));
+		LOG((CLOG_ERR "failed to create a CFRunLoopSourceRef for the quartz event tap"));
 	}
 	CFRunLoopAddSource(CFRunLoopGetCurrent(), m_eventTapRLSR, kCFRunLoopDefaultMode);
 }
@@ -773,7 +826,7 @@ COSXScreen::setClipboard(ClipboardID, const IClipboard* src)
 void
 COSXScreen::checkClipboards()
 {
-	LOG((CLOG_DEBUG1 "checking clipboard"));
+	LOG((CLOG_DEBUG2 "checking clipboard"));
 	if (m_pasteboard.synchronize()) {
 		LOG((CLOG_DEBUG "clipboard changed"));
 		sendClipboardEvent(getClipboardGrabbedEvent(), kClipboardClipboard);
@@ -1074,7 +1127,9 @@ COSXScreen::displayReconfigurationCallback(CGDirectDisplayID displayID, CGDispla
 {
 	COSXScreen* screen = (COSXScreen*)inUserData;
 
-	CGDisplayChangeSummaryFlags mask = kCGDisplayMovedFlag | 
+	// Closing or opening the lid when an external monitor is
+    // connected causes an kCGDisplayBeginConfigurationFlag event
+	CGDisplayChangeSummaryFlags mask = kCGDisplayBeginConfigurationFlag | kCGDisplayMovedFlag | 
 		kCGDisplaySetModeFlag | kCGDisplayAddFlag | kCGDisplayRemoveFlag | 
 		kCGDisplayEnabledFlag | kCGDisplayDisabledFlag | 
 		kCGDisplayMirrorFlag | kCGDisplayUnMirrorFlag | 
@@ -1405,11 +1460,12 @@ COSXScreen::updateScreenShape()
   m_yCenter = (rect.origin.y + rect.size.height) / 2;
 
 	delete[] displays;
-	if (m_isPrimary && !m_isOnScreen) {
-		sendEvent(getShapeChangedEvent());
-	}
+	// We want to notify the peer screen whether we are primary screen or not
+	sendEvent(getShapeChangedEvent());
 
-	LOG((CLOG_DEBUG "screen shape: %d,%d %dx%d on %u %s", m_x, m_y, m_w, m_h, displayCount, (displayCount == 1) ? "display" : "displays"));
+	LOG((CLOG_DEBUG "screen shape: center=%d,%d size=%dx%d on %u %s (%s)",
+         m_x, m_y, m_w, m_h, displayCount,
+         (displayCount == 1) ? "display" : "displays"));
 }
 
 #pragma mark - 
@@ -1722,12 +1778,11 @@ COSXScreen::handleCGInputEventSecondary(CGEventTapProxy proxy,
                         }
                 }
                 if (showCursor) {
-                        LOG((CLOG_DEBUG "Trying to show cursor from local event. (type = %d)", type));
+                        LOG((CLOG_DEBUG "show cursor, event type %d", type));
                         screen->showCursor();
                         screen->m_cursorHidden = false;
                 }
         }
-        LOG((CLOG_DEBUG2 "Local event? (type = %d)", type));
         return event;
 }
 
@@ -1740,7 +1795,7 @@ COSXScreen::handleCGInputEvent(CGEventTapProxy proxy,
 {
 	COSXScreen* screen = (COSXScreen*)refcon;
 	CGPoint pos;
-	
+
 	switch(type) {
 		case kCGEventLeftMouseDown:
 		case kCGEventRightMouseDown:
@@ -1779,10 +1834,10 @@ COSXScreen::handleCGInputEvent(CGEventTapProxy proxy,
 		case kCGEventTapDisabledByTimeout:
 			// Re-enable our event-tap
 			CGEventTapEnable(screen->m_eventTapPort, true);
-			LOG((CLOG_NOTE "Quartz Event tap was disabled by timeout. Re-enabling."));
+			LOG((CLOG_NOTE "quartz event tap was disabled by timeout, re-enabling"));
 			break;
 		case kCGEventTapDisabledByUserInput:
-			LOG((CLOG_ERR "Quartz Event tap was disabled by user input!"));
+			LOG((CLOG_ERR "quartz event tap was disabled by user input"));
 			break;
 		case NX_NULLEVENT:
 			break;
@@ -1793,7 +1848,7 @@ COSXScreen::handleCGInputEvent(CGEventTapProxy proxy,
 		case NX_NUMPROCS:
 			break;
 		default:
-			LOG((CLOG_NOTE "Unknown Quartz Event type: 0x%02x", type));
+			LOG((CLOG_NOTE "unknown quartz event type: 0x%02x", type));
 	}
 	
 	if(screen->m_isOnScreen) {
