@@ -47,6 +47,7 @@ CEvent::Type			CServer::s_switchToScreen     = CEvent::kUnknown;
 CEvent::Type			CServer::s_switchInDirection  = CEvent::kUnknown;
 CEvent::Type			CServer::s_keyboardBroadcast  = CEvent::kUnknown;
 CEvent::Type			CServer::s_lockCursorToScreen = CEvent::kUnknown;
+CEvent::Type			CServer::s_screenSwitched     = CEvent::kUnknown;
 
 CServer::CServer(const CConfig& config, CPrimaryClient* primaryClient, CScreen* screen) :
 	m_primaryClient(primaryClient),
@@ -303,7 +304,7 @@ CServer::adoptClient(CBaseClientProxy* client)
 
 	// name must be in our configuration
 	if (!m_config.isScreen(client->getName())) {
-		LOG((CLOG_WARN "a client with name \"%s\" is not in the map", client->getName().c_str()));
+		LOG((CLOG_WARN "unrecognised client name \"%s\", check server config", client->getName().c_str()));
 		closeClient(client, kMsgEUnknown);
 		return;
 	}
@@ -422,6 +423,13 @@ CServer::getLockCursorToScreenEvent()
 							"CServer::lockCursorToScreen");
 }
 
+CEvent::Type
+CServer::getScreenSwitchedEvent()
+{
+	return EVENTQUEUE->registerTypeOnce(s_screenSwitched,
+							"CServer::screenSwitched");
+}
+
 CString
 CServer::getName(const CBaseClientProxy* client) const
 {
@@ -554,6 +562,10 @@ CServer::switchScreen(CBaseClientProxy* dst,
 		for (ClipboardID id = 0; id < kClipboardEnd; ++id) {
 			m_active->setClipboard(id, &m_clipboards[id].m_clipboard);
 		}
+
+		CServer::CSwitchToScreenInfo* info =
+			CServer::CSwitchToScreenInfo::alloc(m_active->getName());
+		EVENTQUEUE->addEvent(CEvent(CServer::getScreenSwitchedEvent(), this, info));
 	}
 	else {
 		m_active->mouseMove(x, y);
@@ -1222,7 +1234,7 @@ CServer::handleShapeChanged(const CEvent&, void* vclient)
 		return;
 	}
 
-	LOG((CLOG_INFO "screen \"%s\" shape changed", getName(client).c_str()));
+	LOG((CLOG_DEBUG "screen \"%s\" shape changed", getName(client).c_str()));
 
 	// update jump coordinate
 	SInt32 x, y;
