@@ -28,6 +28,17 @@ XScreenOpenFailure::getWhat() const throw()
 }
 
 
+//
+// XScreenXInputFailure
+//
+
+CString
+XScreenXInputFailure::getWhat() const throw()
+{
+	return "";
+}
+
+
 //
 // XScreenUnavailable
 //
