@@ -16,10 +16,10 @@
  */
 
 #include <iostream>
+#include <fstream>
 #include <gtest/gtest.h>
 #include "CArch.h"
 #include "CLog.h"
-#include "LogOutputters.h"
 
 #if SYSAPI_WIN32
 #include "CArchMiscWindows.h"
@@ -35,7 +35,16 @@ void unlock(string lockFile);
 int
 main(int argc, char **argv)
 {
+#if SYSAPI_WIN32
+	// record window instance for tray icon, etc
+	CArchMiscWindows::setInstanceWin32(GetModuleHandle(NULL));
+#endif
+
 	CArch arch;
+	arch.init();
+	
+	CLog log;
+	log.setFilter(kDEBUG2);
 
 	string lockFile;
 	for (int i = 0; i < argc; i++) {
@@ -48,14 +57,6 @@ main(int argc, char **argv)
 		lock(lockFile);
 	}
 
-#if SYSAPI_WIN32
-	// only add std output logger for windows (unix
-	// already outputs to standard streams).
-	CStdLogOutputter stdLogOutputter;
-	CLOG->insert(&stdLogOutputter, true);
-#endif
-
-	CLOG->setFilter(kDEBUG2);
 
 	testing::InitGoogleTest(&argc, argv);
 
@@ -93,7 +94,6 @@ lock(string lockFile)
 	os.close();
 }
 
-
 void
 unlock(string lockFile) 
 {
