@@ -32,8 +32,10 @@ main(int argc, char **argv)
 #endif
 
 	CArch arch;
-
-	CLOG->setFilter(kDEBUG2);
+	arch.init();
+	
+	CLog log;
+	log.setFilter(kDEBUG2);
 
 	testing::InitGoogleTest(&argc, argv);
 
